//
// config_handler.js
// handle the /sigma-config command 
//
const util = require('util');
const { exec } = require('child_process');
const { SIGMA_CLI_PATH } = require('../../config/constants');
const { loadConfig, updateConfig } = require('../../config/config-manager');
const { updateSigmaDatabase } = require('../../services/sigma/sigma_repository_service');
const logger = require('../../utils/logger');

// Promisify exec for async/await usage
const execPromise = util.promisify(exec);

module.exports = (app) => {
  app.command('/sigma-config', async ({ command, ack, respond }) => {
    await ack();
    logger.info(`Sigma config command received: ${command.text}`);
    
    const args = command.text.split(' ');
    
    if (args.length === 0 || args[0] === '') {
      // Display current configuration
      const config = loadConfig();
      logger.info('Displaying current configuration');
      await respond(`Current configuration:\nSIEM: ${config.siem}\nLanguage: ${config.lang}\nOutput: ${config.output}`);
      return;
    }
    
    const configType = args[0];
    
    if (configType === 'update') {
      logger.info('Starting database update from command');
      try {
        await respond('Updating Sigma database... This may take a moment.');
        await updateSigmaDatabase();
        logger.info('Database update completed from command');
        await respond('Sigma database updated successfully');
      } catch (error) {
        logger.error(`Database update failed: ${error.message}`);
        await respond(`Error updating Sigma database: ${error.message}`);
      }
      return;
    }
    
    if (args.length < 2) {
      logger.warn(`Invalid config command format: ${command.text}`);
      await respond(`Invalid command format. Usage: /sigma-config ${configType} [value]`);
      return;
    }
    
    const configValue = args[1];
    const config = loadConfig();
    
    if (configType === 'siem') {
      // Verify the SIEM backend is installed
      logger.info(`Attempting to change SIEM to: ${configValue}`);
      try {
        await execPromise(`${SIGMA_CLI_PATH} list targets | grep ${configValue}`);
        updateConfig('siem', configValue);
        logger.info(`SIEM configuration updated to: ${configValue}`);
        await respond(`SIEM configuration updated to: ${configValue}`);
      } catch (error) {
        logger.error(`SIEM backend '${configValue}' not found or not installed`);
        await respond(`Error: SIEM backend '${configValue}' not found or not installed. Please install it with: sigma plugin install ${configValue}`);
      }
    } else if (configType === 'lang') {
      logger.info(`Changing language to: ${configValue}`);
      updateConfig('lang', configValue);
      await respond(`Language configuration updated to: ${configValue}`);
    } else if (configType === 'output') {
      // Check if output format is supported by the current backend
      logger.info(`Attempting to change output format to: ${configValue}`);
      try {
        await execPromise(`${SIGMA_CLI_PATH} list formats ${config.siem} | grep ${configValue}`);
        updateConfig('output', configValue);
        logger.info(`Output configuration updated to: ${configValue}`);
        await respond(`Output configuration updated to: ${configValue}`);
      } catch (error) {
        logger.error(`Output format '${configValue}' not supported by SIEM backend '${config.siem}'`);
        await respond(`Error: Output format '${configValue}' not supported by SIEM backend '${config.siem}'. Run 'sigma list formats ${config.siem}' to see available formats.`);
      }
    } else {
      logger.warn(`Unknown configuration type: ${configType}`);
      await respond(`Unknown configuration type: ${configType}. Available types: siem, lang, output, update`);
    }
  });
};